@@ -42,10 +42,12 @@ module MetasploitModule
       self.arch = actual_arch
     end
 
-    if datastore['PAYLOADFILE']
-      IO.read(datastore['PAYLOADFILE'])
-    elsif datastore['PAYLOADSTR']
+    if datastore['PAYLOADSTR']
       datastore['PAYLOADSTR']
+    elsif datastore['PAYLOADFILE']
+      IO.read(datastore['PAYLOADFILE'])
+    else
+      ''
     end
   end
 
